use std::fmt::Display;

use viperid::VResult;
use wasmi::{core::{HostError, Trap}, Caller, Linker};

use crate::{executor::ExecutorState, HOSTED_DEVICE};

pub(crate) fn integrate(linker: &mut Linker<ExecutorState>) -> VResult<()> {
    linker.func_wrap("viperid", "Pset", pset)?;

    linker.func_wrap("viperid", "YieldFrame", yield_frame)?;

    Ok(())
}

fn pset(mut _caller: Caller<ExecutorState>, x: i32, y: i32, color: i32) -> Result<(), Trap> {
    HOSTED_DEVICE.with(|dev| {
        dev.shared.screen.pset(x, y, (color & 0xff) as u8);
        Ok(())
    })
}

fn yield_frame(_caller: Caller<ExecutorState>) -> Result<(), Trap> {
    Err(Trap::from(YieldFrame {}))
}

#[derive(Debug)]
pub(crate) struct YieldFrame {

}

impl Display for YieldFrame {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        f.write_str("<device wants to display a frame>")
    }
}

impl HostError for YieldFrame {

}
